#!/usr/bin/env python3

# This amulet test deploys the bundles.yaml file in this directory.

import os
import unittest
import yaml
import amulet
import requests

seconds_to_wait = 960


class BundleTest(unittest.TestCase):
    """ Create a class for testing the charm in the unit test framework. """
    @classmethod
    def setUpClass(cls):
        """ Set up an amulet deployment using the bundle. """
        d = amulet.Deployment()
        bundle_path = os.path.join(os.path.dirname(__file__), 'bundles.yaml')
        with open(bundle_path, 'r') as bundle_file:
            contents = yaml.safe_load(bundle_file)
            d.load(contents)
        d.setup(seconds_to_wait, cleanup=False)
        d.sentry.wait(seconds_to_wait)
        cls.d = d

    def test_deployed(self):
        """ Test to see if the bundle deployed successfully. """
        self.assertTrue(self.d.deployed)

    def test_port_change(self):
        """ Change kibana port and validate the change happened """
        self.d.configure('kibana', {'port': 81})
        self.d.sentry.wait()

        url = 'http://%s:81/' % kibana_unit.info['public-address']
        r = requests.get(url)
        assert r.ok

    def test_web_interface():

        elasticsearch_unit.run("""
            curl -X POST 'http://127.0.0.1:9200/person/1' -d '{
            "info" : {
                    "height" : 2,
                    "width" : 20
            }
            }'
        """)

        time.sleep(3)

        # Ensure the kibana unit can query ES via localhost (it proxies local 9200
        # queries to ES)
        url = 'http://127.0.0.1:9200/_all/_search'
        result = kibana_unit.run(
            "curl -XGET {url} -d '@11-scale-elastic-query.json'".format(url=url))
        if not result[1] == 0:
            amulet.raise_status(
                amulet.FAIL, msg="Error performing search. Error code: {}. "
                                 "Result:\n{}".format(result[1], result[0]))
        try:
            result = json.loads(result[0])
            if result['hits']['total'] != 1:
                amulet.raise_status(amulet.FAIL,
                                    msg="Error inserting value in elasticsearch.")
        except ValueError:
            exc_type, value, traceback = sys.exc_info()
            exc_text = "Exception: (%s) %s. %s" % (value, exc_type, traceback)
            amulet.raise_status(
                amulet.FAIL, msg="Value returned not a valid json.\n{}\n{}".format(
                    result[0], exc_text))


if __name__ == '__main__':
    unittest.main()
